from xml.etree.ElementTree import Element, SubElement, tostring
from xml.dom import minidom
from phoebusgen._shared_property_helpers import _SharedPropertyFunctions


class Screen(object):
    def __init__(self, name, f_name=None):
        self.bob_file = f_name
        self.root = Element('display', version='2.0.0')
        name_child = SubElement(self.root, 'name')
        name_child.text = name
        self.shared_functions = _SharedPropertyFunctions(self.root)

    def write_screen(self, file_name=None):
        rough_string = tostring(self.root, 'utf-8')
        reparse_xml = minidom.parseString(rough_string)
        if file_name is None:
            file_name = self.bob_file
        if file_name is None:
            print('Output Phoebus file name is not set! First set bob_file or use file_name parameter')
            return False
        else:
            with open(file_name, 'w') as f:
                reparse_xml.writexml(f, indent="  ", addindent="  ", newl="\n", encoding="UTF-8")
            return True

    def find_widget(self, widget_tag_name):
        elements = self.root.findall(widget_tag_name)
        if len(elements) > 1:
            print('Warning, more than one element of the same tag! Returning a list')
            return elements
        elif len(elements) == 0:
            return None
        else:
            return elements[0]

    def add_widget(self, elem):
        if type(elem) == list:
            for e in elem:
                self.root.append(e.root)
        else:
            self.root.append(elem.root)

    def width(self, val):
        self.shared_functions.number_property('width', val)

    def height(self, val):
        self.shared_functions.number_property('height', val)

    def macro(self, name, val):
        self.shared_functions.add_macro(name, val)

    def background_color(self, red, green, blue, alpha=255):
        e = self.shared_functions.create_element('background_color')
        self.shared_functions.create_color_element(e, None, red, green, blue, alpha)

    def predefined_background_color(self, name):
        e = self.shared_functions.create_element('background_color')
        self.shared_functions.create_color_element(e, name, None, None, None, None)

    # From: https://pymotw.com/3/xml.etree.ElementTree/create.html
    def prettify(self, elem):
        """Return a pretty-printed XML string for the Element.
        """
        rough_string = tostring(elem, 'utf-8')
        reparse_xml = minidom.parseString(rough_string)
        return reparse_xml.toprettyxml(indent="  ", newl="\n")

    def __str__(self):
        return self.prettify(self.root)

    def __repr__(self):
        return self.prettify(self.root)


if __name__ == '__main__':
    s = Screen('DisplayTest')



"phoebusgen package -- test_widget"

import phoebusgen.widget
import phoebusgen.screen

from os import path as _path
from sys import platform as _platform
import re as _re
from enum import Enum as _enum

_curr_path = _path.dirname(__file__)
_color_def = _curr_path + '/config/color.def'
_font_def = _curr_path + '/config/font.def'
_classes_bcf = _curr_path + '/config/classes.bcf'
_local_color_def = _path.expanduser('~/.phoebusgen/color.def')
_local_font_def = _path.expanduser('~/.phoebusgen/font.def')
if _path.isfile(_local_color_def):
    _color_def = _local_color_def
if _path.isfile(_local_font_def):
    _font_def = _local_font_def

def _update_color_def(file_path):
    #print('Using color.def file at: {}'.format(file_path))
    predefined_colors = {}
    if not _path.isfile(file_path):
        print('File at this path does not exist: {}'.format(file_path))
    with open(file_path, 'r') as color_file:
        for line in color_file:
            line = line.partition('#')[0].rstrip()
            if line != "":
                color, value = line.split('=')
                color = color.strip()
                vals = [v.strip() for v in value.split(',')]
                if len(vals) == 1:
                    predefined_colors[color] = predefined_colors[vals[0]]
                else:
                    if len(vals) == 4:
                        alpha = vals[3]
                    else:
                        alpha = 255
                    predefined_colors[color] = {'name': color, 'red': str(vals[0]), 'green': str(vals[1]),
                                                'blue': str(vals[2]), 'alpha': str(alpha)}
    return predefined_colors

def _update_font_def(file_path):
    if not _path.isfile(file_path):
        print('File at this path does not exist: {}'.format(file_path))
    with open(file_path, 'r') as font_file:
        predefined_fonts = {}
        os = _platform.lower()
        if 'linux' in os:
            os = 'linux'
        elif 'darwin' in os:
            os = 'macosx'
        elif 'win' in os:
            os = 'windows'
        for line in font_file:
            line = line.partition('//')[0].rstrip()
            if line != "":
                font, value = line.split('=')
                font = font.strip()
                os_name = _re.search('\(([^)]+)', font)
                if os_name:
                    os_name = os_name.group(1)
                    if os_name != os:
                        continue
                    else:
                        font = font.replace('(', '')
                        font = font.replace(')', '')
                        font = font.replace(os, '')
                vals = [v.strip() for v in value.split('-')]
                if len(vals) == 1:
                    predefined_fonts[font] = predefined_fonts[vals[0].strip('@')]
                else:
                    family = vals[0]
                    style = vals[1]
                    size = vals[2]
                    if style.lower() == 'regular':
                        style = 'REGULAR'
                    elif style.lower() == 'bold':
                        style = 'BOLD'
                    elif style.lower() == 'italic':
                        style = 'ITALIC'
                    elif style.lower() == 'bold italic':
                        style = 'BOLD_ITALIC'
                    predefined_fonts[font.replace(' ', '')] = {'name': font, 'family': family, 'style': style, 'size': size}
        return predefined_fonts



_predefined_colors = _update_color_def(_color_def)
colors = _enum('colors', _predefined_colors)
_predefined_fonts = _update_font_def(_font_def)
fonts = _enum('fonts', _predefined_fonts)



class _PVName(object):
    def pv_name(self, name):
        self._prop_factory.add_pv_name(name)


class _Font(object):
    def predefined_font(self, name):
        self._prop_factory.add_named_font(name)

    def font_family(self, family):
        self._prop_factory.add_font_family(family)

    def font_size(self, size):
        self._prop_factory.add_font_size(size)

    def font_style_bold(self):
        self._prop_factory.add_font_style(self._prop_factory.FontStyle.bold)

    def font_style_italic(self):
        self._prop_factory.add_font_style(self._prop_factory.FontStyle.italic)

    def font_style_bold_italic(self):
        self._prop_factory.add_font_style(self._prop_factory.FontStyle.bold_and_italic)

    def font_style_regular(self):
        self._prop_factory.add_font_style(self._prop_factory.FontStyle.regular)

class _ForegroundColor(object):
    def predefined_foreground_color(self, name):
        self._prop_factory.add_foreground_color(name, None, None, None, None)

    def foreground_color(self, red, green, blue, alpha=255):
        self._prop_factory.add_foreground_color(None, red, green, blue, alpha)


class _BackgroundColor(object):
    def predefined_background_color(self, name):
        self._prop_factory.add_background_color(name, None, None, None, None)

    def background_color(self, red, green, blue, alpha=255):
        self._prop_factory.add_background_color(None, red, green, blue, alpha)


class _Transparent(object):
    def transparent(self, transparent=False):
        self._prop_factory.add_transparent(transparent)


class _Format(object):
    def format(self, format):
        self._prop_factory.add_format(format)


class _Precision(object):
    def precision(self, val):
        self._prop_factory.add_precision(val)

class _ShowUnits(object):
    def show_units(self, show=True):
        self._prop_factory.add_show_units(show)


class _HorizontalAlignment(object):
    def horizontal_alignment_left(self):
        self._prop_factory.add_horizontal_alignment(self._prop_factory.HorizontalAlignment.left)

    def horizontal_alignment_center(self):
        self._prop_factory.add_horizontal_alignment(self._prop_factory.HorizontalAlignment.center)

    def horizontal_alignment_right(self):
        self._prop_factory.add_horizontal_alignment(self._prop_factory.HorizontalAlignment.right)

class _VerticalAlignment(object):
    def vertical_alignment_top(self):
        self._prop_factory.add_vertical_alignment(self._prop_factory.VerticalAlignment.top)

    def vertical_alignment_middle(self):
        self._prop_factory.add_vertical_alignment(self._prop_factory.VerticalAlignment.middle)

    def vertical_alignment_bottom(self):
        self._prop_factory.add_vertical_alignment(self._prop_factory.VerticalAlignment.bottom)


class _WrapWords(object):
    def wrap_words(self, wrap=True):
        self._prop_factory.add_wrap_words(wrap)


class _Text(object):
    def text(self, text):
        self._prop_factory.add_text(text)


class _AutoSize(object):
    def auto_size(self, auto=True):
        self._prop_factory.add_auto_size(auto)


# 0, 90, 180, -90
# text update, label, action button
class _RotationStep(object):
    def rotation_step_0(self):
        self._prop_factory.add_rotation_step(self._prop_factory.RotationStep.zero)

    def rotation_step_90(self):
        self._prop_factory.add_rotation_step(self._prop_factory.RotationStep.ninety)

    def rotation_step_180(self):
        self._prop_factory.add_rotation_step(self._prop_factory.RotationStep.one_hundred_eighty)

    def rotation_step_negative_90(self):
        self._prop_factory.add_rotation_step(self._prop_factory.RotationStep.negative_ninety)

class _Border(object):
    def border_width(self, width):
        self._prop_factory.add_border_width(width)

    def predefined_border_color(self, name):
        self._prop_factory.add_border_color(name, None, None, None, None)

    def border_color(self, red, green, blue, alpha=255):
        self._prop_factory.add_border_color(None, red, green, blue, alpha)


class _Macro(object):
    def macro(self, name, val):
        self._prop_factory.add_macro(name, val)


class _Bit(object):
    def bit(self, val=-1):
        self._prop_factory.add_bit(val)


class _OffColor(object):
    def predefined_off_color(self, name):
        self._prop_factory.add_off_color(name, None, None, None, None)

    def off_color(self, red, green, blue, alpha=255):
        self._prop_factory.add_off_color(None, red, green, blue, alpha)

class _Off(_OffColor):
    def off_label(self, label):
        self._prop_factory.add_off_label(label)

class _OffImage(_Off):
    def off_image(self, val):
        self._prop_factory.add_off_image(val)

class _OnColor(object):
    def predefined_on_color(self, name):
        self._prop_factory.add_on_color(name, None, None, None, None)

    def on_color(self, red, green, blue, alpha=255):
        self._prop_factory.add_on_color(None, red, green, blue, alpha)

class _On(_OnColor):
    def on_label(self, label):
        self._prop_factory.add_on_label(label)

class _OnImage(_On):
    def on_image(self, val):
        self._prop_factory.add_on_image(val)


# Arc, Ellipse, Polygon, Polyline, Rectangle, LED, LED Multi State
class _LineColor(object):
    def predefined_line_color(self, name):
        self._prop_factory.add_line_color(name, None, None, None, None)

    def line_color(self, red, green, blue, alpha=255):
        self._prop_factory.add_line_color(None, red, green, blue, alpha)


# Arc, Ellipse, Polygon, Polyline, Rectangle
class _LineWidth(object):
    def line_width(self, width):
        self._prop_factory.add_line_width(width)


# Rectangle
class _Corner(object):
    def corner_width(self, width):
        self._prop_factory.add_corner_width(width)

    def corner_height(self, height):
        self._prop_factory.add_corner_height(height)


class _Square(object):
    def square(self, val):
        self._prop_factory.add_square(val)


class _LabelsFromPV(object):
    def labels_from_pv(self, val):
        self._prop_factory.add_labels_from_pv(val)


class _AlarmBorder(object):
    def alarm_border(self, val):
        self._prop_factory.add_alarm_border(val)


class _Enabled(object):
    def enabled(self, val):
        self._prop_factory.add_enabled(val)


class _Confirmation(object):
    def confirmation_dialog(self, message, password=None):
        self._prop_factory.add_confirmation_dialog(True)
        self._prop_factory.add_confirmation_message(message)
        if password is not None:
            self._prop_factory.add_password(password)

    def disable_confirmation_dialog(self):
        self._prop_factory.add_confirmation_dialog(False)


class _MultiLine(object):
    def multi_line(self, val):
        self._prop_factory.add_multi_line(val)


class _Angle(object):
    def angle_start(self, val):
        self._prop_factory.add_angle_start(val)

    def angle_size(self, val):
        self._prop_factory.add_angle_size(val)

# picture, symbol
class _Rotation(object):
    def rotation(self, val):
        self._prop_factory.add_rotation(val)


class _File(object):
    def file(self, val):
        self._prop_factory.add_file(val)

class _StretchToFit(object):
    def stretch_to_fit(self, val):
        self._prop_factory.add_stretch_to_fit(val)

class _Actions(object):
    def action_execute_as_one(self, val):
        self._prop_factory.add_action_execute_as_one(val)

    def action_open_display(self, file, target, description=None):
        # still needs macros
        if description is None:
            description = 'Open Display'
        possible_targets = ['tab', 'replace', 'window']
        if target.lower() not in possible_targets:
            print('Target must be one of {}, not: {}'.format(possible_targets, target))
            return
        args = {'file': file, 'target': target.lower()}
        self._prop_factory.add_action('open_display', description, args)

    def action_write_pv(self, pv_name, value, description=None):
        if description is None:
            description = 'Write PV'
        args = {'pv_name': pv_name, 'value': value}
        self._prop_factory.add_action('write_pv', description, args)

    def action_execute_command(self, command, description=None):
        if description is None:
            description = 'Execute Command'
        args = {'command': command}
        self._prop_factory.add_action('command', description, args)

    def action_open_file(self, file, description=None):
        if description is None:
            description = 'Open File'
        args = {'file': file}
        self._prop_factory.add_action('open_file', description, args)

    def action_open_webpage(self, url, description=None):
        if description is None:
            description = 'Open Webpage'
        args = {'url': url}
        self._prop_factory.add_action('open_webpage', description, args)

class _Label(object):
    def label(self, val):
        self._prop_factory.add_label(val)

class _Horizontal(object):
    def horizontal(self, val):
        self._prop_factory.add_horizontal(val)

class _Items(object):
    def item(self, val):
        self._prop_factory.add_item(val)

class _ItemsFromPV(object):
    def items_from_pv(self, val):
        self._prop_factory.add_items_from_pv(val)

class _StartBit(object):
    pass

class _NumBits(object):
    pass

class _ReverseBits(object):
    pass

class _States(object):
    pass

class _Fallback(object):
    pass

class _ShowValueTip(object):
    def show_value_tip(self, val):
        self._prop_factory.add_show_value_tip(val)

class _BarLength(object):
    def bar_length(self, val):
        self._prop_factory.add_bar_length(val)

class _ShowValue(object):
    def show_value(self, val):
        self._prop_factory.add_show_value(val)

class _ShowLimits(object):
    def show_limits(self, val):
        self._prop_factory.add_show_limits(val)

class _LimitsFromPV(object):
    def limits_from_pv(self, val):
        self._prop_factory.add_limits_from_pv(val)

class _MinMax(object):
    def minimum(self, val):
        self._prop_factory.add_minimum(val)

    def maximum(self, val):
        self._prop_factory.add_maximum(val)

class _NeedleColor(object):
    def needle_color(self, red, green, blue, alpha=255):
        self._prop_factory.add_needle_color(None, red, green, blue, alpha)

    def predefined_needle_color(self, name):
        self._prop_factory.add_needle_color(name, None, None, None, None)

class _KnobColor(object):
    def knob_color(self, red, green, blue, alpha=255):
        self._prop_factory.add_knob_color(None, red, green, blue, alpha)

    def predefined_knob_color(self, name):
        self._prop_factory.add_knob_color(name, None, None, None, None)

class _FillColor(object):
    def fill_color(self, red, green, blue, alpha=255):
        self._prop_factory.add_fill_color(None, red, green, blue, alpha)

    def predefined_fill_color(self, name):
        self._prop_factory.add_fill_color(name, None, None, None, None)

class _EmptyColor(object):
    def empty_color(self, red, green, blue, alpha=255):
        self._prop_factory.add_empty_color(None, red, green, blue, alpha)

    def predefined_empty_color(self, name):
        self._prop_factory.add_empty_color(name, None, None, None, None)

class _ScaleVisible(object):
    def scale_visible(self, val):
        self._prop_factory.add_scale_visible(val)

class _ShowLED(object):
    def show_led(self, val):
        self._prop_factory.add_show_led(val)

class _Mode(object):
    def mode_toggle(self):
        self._prop_factory.add_mode(self._prop_factory.Mode.toggle)

    def mode_push(self):
        self._prop_factory.add_mode(self._prop_factory.Mode.push)

    def mode_push_inverted(self):
        self._prop_factory.add_mode(self._prop_factory.Mode.push_inverted)

class _Style(object):
    def style_group_box(self):
        self._prop_factory.add_style(self._prop_factory.GroupStyle.group_box)

    def style_title_bar(self):
        self._prop_factory.add_style(self._prop_factory.GroupStyle.title_bar)

    def style_line(self):
        self._prop_factory.add_style(self._prop_factory.GroupStyle.line)

    def no_style(self):
        self._prop_factory.add_style(self._prop_factory.GroupStyle.none)


class _ResizeBehavior(object):
    def no_resize(self):
        self._prop_factory.add_resize_behavior(self._prop_factory.Resize.no_resize)

    def size_content_to_fit_widget(self):
        self._prop_factory.add_resize_behavior(self._prop_factory.Resize.size_content_to_fit_widget)

    def size_widget_to_match_content(self):
        self._prop_factory.add_resize_behavior(self._prop_factory.Resize.size_widget_to_match_content)

    def stretch_content_to_fit_widget(self):
        self._prop_factory.add_resize_behavior(self._prop_factory.Resize.stretch_content_to_fit_widget)

    def crop_content(self):
        self._prop_factory.add_resize_behavior(self._prop_factory.Resize.crop_content)


class _GroupName(object):
    def group_name(self, val):
        self._prop_factory.add_group_name(val)


class _Structure(object):
    def add_widget(self, elem):
        if type(elem) == list:
            for e in elem:
                self._prop_factory.root.append(e.root)
        else:
            self._prop_factory.root.append(elem.root)

class _URL(object):
    def url(self, url):
        self._prop_factory.add_url(url)

class _ShowToolbar(object):
    def show_toolbar(self, val):
        self._prop_factory.add_show_toolbar(val)

class _ButtonsOnLeft(object):
    def buttons_on_left(self, val):
        self._prop_factory.add_buttons_on_left(val)

class _Increment(object):
    def increment(self, val):
        self._prop_factory.add_increment(val)

class _FileComponent(object):
    def file_component_full_path(self):
        self._prop_factory.add_file_component(self._prop_factory.FileComponent.full_path)

    def file_component_directory(self):
        self._prop_factory.add_file_component(self._prop_factory.FileComponent.directory)

    def file_component_name_and_extension(self):
        self._prop_factory.add_file_component(self._prop_factory.FileComponent.name_and_extension)

    def file_component_base_name(self):
        self._prop_factory.add_file_component(self._prop_factory.FileComponent.base_name)

class _Editable(object):
    def editable(self, val):
        self._prop_factory.add_editable(val)

class _SelectedColor(object):
    def selected_color(self, red, green, blue, alpha=255):
        self._prop_factory.add_selected_color(None, red, green, blue, alpha)

    def predefined_selected_color(self, name):
        self._prop_factory.add_selected_color(name, None, None, None, None)

class _SelectionValuePV(object):
    def selection_value_pv(self, val):
        self._prop_factory.add_selection_value_pv(val)


from xml.etree.ElementTree import Element, SubElement
from enum import Enum

class _SharedPropertyFunctions(object):
    def __init__(self, root_element):
        self.root = root_element
        from phoebusgen import colors, _predefined_colors, fonts, _predefined_fonts
        self.predefined_colors = _predefined_colors
        self.predefined_fonts = _predefined_fonts
        self.colors = colors
        self.fonts = fonts

    def add_macro(self, name, val):
        root_macro = self.root.find('macros')
        if root_macro is None:
            root_macro = SubElement(self.root, 'macros')
        macro = SubElement(root_macro, name)
        macro.text = str(val)

    def generic_property(self, prop_type, val=None):
        self.root.append(self.create_element(prop_type, val))

    def integer_property(self, prop_type, val=None):
        if type(val) == int or type(val) == float:
            self.generic_property(prop_type, int(val))
        else:
            print('Property {} must be an integer! Not: {}'.format(prop_type, val))

    def number_property(self, prop_type, val):
        if type(val) == int or type(val) == float:
            self.generic_property(prop_type, val)
        else:
            print('Property {} must be a number! Not: {}'.format(prop_type, val))

    def boolean_property(self, prop_type, val):
        if type(val) == bool:
            self.generic_property(prop_type, str(val).lower())
        elif type(val) == int:
            self.generic_property(prop_type, str(bool(val)).lower())
        elif val.lower() == 'true' or val.lower() == 'false':
            self.generic_property(prop_type, val.lower())
        else:
            print('Property {} must be a boolean value! Not: {}'.format(prop_type, val))

    def create_element(self, prop_type, val=None):
        element = self.root.find(prop_type)
        if element is not None:
            self.root.remove(element)
        element = Element(prop_type)
        if val is not None:
            if type(val) == bool:
                element.text = str(val).lower()
            else:
                element.text = str(val)
        return element

    def valid_rgb_value(self, val):
        try:
            val = int(val)
        except ValueError:
            print('Color RGB value must be a number! Not: {}'.format(val))
            return False
        if 0 <= val <= 255:
            return True
        else:
            print('Color RGB must be between 0 and 255')
            return False

    def create_color_element(self, root_color_elem, name, red, green, blue, alpha):
        sub_e = self.create_element('color')
        if name is None:
            for color in [red, green, blue, alpha]:
                if not self.valid_rgb_value(color):
                    return
            sub_e.attrib = {'red': str(red), 'blue': str(blue), 'green': str(green), 'alpha': str(alpha)}
        else:
            if isinstance(name, Enum):
                sub_e.attrib['name'] = name.name
                sub_e.attrib = name.value
            elif isinstance(name, dict):
                sub_e.attrib = name
            elif isinstance(name, str):
                color_attrib = self.predefined_colors.get(name)
                if color_attrib is None:
                    print('Color name is undefined')
                    return
                sub_e.attrib = color_attrib
                sub_e.attrib['name'] = name
            else:
                print('Predefined color input must be phoebusgen.colors.<named-color>, not: {} of type: {}'.format(name, type(name)))
                return
        root_color_elem.append(sub_e)
        self.root.append(root_color_elem)

    def create_named_font_elemet(self, name):
        root_font_elem = self.create_element('font')
        child_font_elem = self.create_element('font')
        if isinstance(name, Enum):
            font_attrib = name.value
        elif isinstance(name, dict):
            font_attrib = name
        elif isinstance(name, str):
            font_attrib = self.predefined_fonts.get(name.lower())
            if font_attrib is None:
                print('Font name is undefined')
                return
            font_attrib['style'] = font_attrib['style'].upper()
        else:
            print('Predefined font input must be phoebusgen.fonts.<named-color>, not: {} of type: {}'.format(name, type(name)))
            return
        child_font_elem.attrib = font_attrib
        root_font_elem.append(child_font_elem)
        self.root.append(root_font_elem)

if __name__ == '__main__':
    pass



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="phoebusgen",
    version="1.0.0",
    author="Tynan Ford",
    author_email="tford@lbl.gov",
    description="Control screen generator for Phoebus",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/tynanford/phoebusgen",
    packages=setuptools.find_packages(),
    include_package_data=True,
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License v2 (GPLv2)",
        "Operating System :: OS Independent",
        "Development Status :: 5 - Production/Stable",
        "Topic :: Software Development :: Code Generators"
    ],
    python_requires='>=3.5',
)




from phoebusgen.widget.widget import _Widget
from phoebusgen.widget import property_stubs as _p

# Displays
class Arc(_Widget, _p._Macro, _p._Angle, _p._LineWidth, _p._LineColor, _p._BackgroundColor, _p._Transparent):
    """ Arc widget, req: none """
    def __init__(self, name, x, y, width, height):
        _Widget.__init__(self, 'arc', name, x, y, width, height)

class Ellipse(_Widget, _p._Macro, _p._LineWidth, _p._LineColor, _p._BackgroundColor, _p._Transparent):
    """ Ellipse widget, req: none """
    def __init__(self, name, x, y, width, height):
        _Widget.__init__(self, 'ellipse', name, x, y, width, height)

class Label(_Widget, _p._Text, _p._Macro, _p._Font, _p._ForegroundColor, _p._BackgroundColor, _p._Transparent, _p._HorizontalAlignment,
            _p._VerticalAlignment, _p._RotationStep, _p._WrapWords, _p._AutoSize, _p._Border):
    """ Label widget, req: text """
    def __init__(self, name, text, x, y, width, height):
        _Widget.__init__(self, 'label', name, x, y, width, height)
        self.text(text)

class Picture(_Widget, _p._Macro, _p._File, _p._StretchToFit, _p._Rotation):
    def __init__(self, name, file, x, y, width, height):
        _Widget.__init__(self, 'picture', name, x, y, width, height)
        self.file(file)

# points
#class Polygon(_Widget, _p._Macro, _p._LineWidth, _p._LineColor, _p._BackgroundColor, _p._Points):
#    pass

# line style, arrows, arrow length, points
#class Polyline(_Widget, _p._Macro, _p._LineWidth, _p._LineColor, _p._LineStyle, _p._Arrow, _p.Points):
#    pass

class Rectangle(_Widget, _p._Macro, _p._LineWidth, _p._LineColor, _p._BackgroundColor, _p._Transparent, _p._Corner):
    """ Rectangle widget, req: none """
    def __init__(self, name, x, y, width, height):
        _Widget.__init__(self, 'rectangle', name, x, y, width, height)

# Monitors
# Start bit, number of bits, reverse bits, labels
#class ByteMonitor(_Widget, _p._PVName, _p._StartBit, _p._NumBits, _p._ReverseBits, _p._Horizontal, _p._Square, _p._OffColor, _p._OnColor,
#                  _p._ForegroundColor, _p._Font, _p._Labels, _p._AlarmBorder):
#    pass

class LED(_Widget, _p._PVName, _p._Bit, _p._Off, _p._On, _p._Font, _p._ForegroundColor, _p._LineColor,
          _p._Square, _p._LabelsFromPV, _p._AlarmBorder):
    """ LED widget, req: None """
    def __init__(self, name, pv_name, x, y, width, height):
        _Widget.__init__(self, 'led', name, x, y, width, height)
        self.pv_name(pv_name)

# States, Fallback
#class LEDMultiState(_Widget, _p._PVName, _p._Font, _p._ForegroundColor, _p._LineColor, _p._Square,
#                    _p._AlarmBorder, _p._States, _p._Fallback):
#    pass

class Meter(_Widget, _p._PVName, _p._ForegroundColor, _p._BackgroundColor, _p._Font, _p._Format,
            _p._Precision, _p._ShowValue, _p._ShowUnits, _p._ShowLimits, _p._AlarmBorder,
            _p._LimitsFromPV, _p._MinMax, _p._NeedleColor, _p._KnobColor):
    def __init__(self, name, pv_name, x, y, width, height):
        _Widget.__init__(self, 'meter', name, x, y, width, height)
        self.pv_name(pv_name)

class ProgressBar(_Widget, _p._PVName, _p._FillColor, _p._BackgroundColor, _p._Horizontal,
                  _p._AlarmBorder, _p._LimitsFromPV, _p._MinMax):
    def __init__(self, name, pv_name, x, y, width, height):
        _Widget.__init__(self, 'progressbar', name, x, y, width, height)
        self.pv_name(pv_name)

# Symbols, InitialIndex, showindex, arrayindex, preserve ratio
#class Symbol(_Widget, _p._Symbols, _p._PVName, _p._BackgroundColor, _p._InitialIndex,
#             _p._Rotation, _p._ShowIndex, _p._Transparent, _p._AlarmBorder, _p._ArrayIndex,
#             _p._AutoSize, _p._Enabled, _p._PreserveRatio):

# columns, selectrows, selectionpv
#class Table(_Widget, _p._PVName, _p._Font, _p._ForegroundColor, _p._BackgroundColor, _p._ShowToolbar,
#            _p._Columns, _p._AlarmBorder, _p._Editable, _p._SelectRows, _p._SelectionPV):

class Tank(_Widget, _p._PVName, _p._Font, _p._ForegroundColor, _p._BackgroundColor,
           _p._FillColor, _p._EmptyColor, _p._ScaleVisible, _p._AlarmBorder, _p._LimitsFromPV,
           _p._MinMax):
    def __init__(self, name, pv_name, x, y, width, height):
        _Widget.__init__(self, 'tank', name, x, y, width, height)
        self.pv_name(pv_name)

# Symbols, arrayindex
#class TextSymbol(_Widget, _p._PVName, _p._Symbols, _p._Font, _p._ForegroundColor, _p._BackgroundColor,
#                 _p._Transparent, _p._HorizontalAlignment, _p._VerticalAlignment, _p._Rotation,
#                 _p._WrapWords, _p._AlarmBorder, _p._ArrayIndex, _p._Enabled):
#    pass

class TextUpdate(_Widget, _p._PVName, _p._Font, _p._ForegroundColor, _p._BackgroundColor, _p._Transparent,
                 _p._Format, _p._Precision, _p._ShowUnits, _p._HorizontalAlignment, _p._VerticalAlignment, _p._WrapWords,
                 _p._RotationStep, _p._Border):
    """ Text Update widget, req: pv name """
    def __init__(self, name, pv_name, x, y, width, height):
        _Widget.__init__(self, 'textupdate', name, x, y, width, height)
        self.pv_name(pv_name)

class Thermometer(_Widget, _p._PVName, _p._FillColor, _p._AlarmBorder, _p._LimitsFromPV, _p._MinMax):
    def __init__(self, name, pv_name, x, y, width, height):
        _Widget.__init__(self, 'thermometer', name, x, y, width, height)
        self.pv_name(pv_name)

# Controls
class ActionButton(_Widget, _p._PVName, _p._Actions, _p._Text, _p._Font, _p._ForegroundColor, _p._BackgroundColor,
                   _p._Transparent, _p._RotationStep, _p._Enabled, _p._Confirmation):
    """ Action button widget, req: text, pv name """
    def __init__(self, name, text, pv_name, x, y, width, height):
        _Widget.__init__(self, 'action_button', name, x, y, width, height)
        self.pv_name(pv_name)
        self.text(text)

class BooleanButton(_Widget, _p._PVName, _p._Bit, _p._OffImage, _p._OnImage, _p._ShowLED, _p._Font, _p._ForegroundColor, _p._BackgroundColor,
                    _p._LabelsFromPV, _p._AlarmBorder, _p._Enabled, _p._Mode, _p._Confirmation):
    def __init__(self, name, pv_name, x, y, width, height):
        _Widget.__init__(self, 'bool_button', name, x, y, width, height)
        self.pv_name(pv_name)

class CheckBox(_Widget, _p._PVName, _p._Bit, _p._Label, _p._Font, _p._ForegroundColor, _p._AutoSize,
               _p._AlarmBorder, _p._Confirmation):
    def __init__(self, name, label, pv_name, x, y, width, height):
        _Widget.__init__(self, 'checkbox', name, x, y, width, height)
        self.pv_name(pv_name)
        self.label(label)

class ChoiceButton(_Widget, _p._PVName, _p._Font, _p._ForegroundColor, _p._BackgroundColor, _p._SelectedColor, _p._Horizontal,
                   _p._AlarmBorder, _p._Items, _p._ItemsFromPV, _p._Confirmation):
    def __init__(self, name, pv_name, x, y, width, height):
        _Widget.__init__(self, 'choice', name, x, y, width, height)
        self.pv_name(pv_name)

class ComboBox(_Widget, _p._PVName, _p._Font, _p._ForegroundColor, _p._BackgroundColor, _p._AlarmBorder, _p._Items,
               _p._ItemsFromPV, _p._Editable, _p._Enabled, _p._Confirmation):
    def __init__(self, name, pv_name, x, y, width, height):
        _Widget.__init__(self, 'combo', name, x, y, width, height)
        self.pv_name(pv_name)


class FileSelector(_Widget, _p._PVName, _p._FileComponent, _p._AlarmBorder, _p._Enabled):
    def __init__(self, name, pv_name, x, y, width, height):
        _Widget.__init__(self, 'fileselector', name, x, y, width, height)
        self.pv_name(pv_name)

class RadioButton(_Widget, _p._PVName, _p._Font, _p._ForegroundColor, _p._Horizontal, _p._AlarmBorder,
                  _p._Items, _p._ItemsFromPV, _p._Enabled, _p._Confirmation):
    def __init__(self, name, pv_name, x, y, width, height):
        _Widget.__init__(self, 'radio', name, x, y, width, height)
        self.pv_name(pv_name)

# showscale, showminorticks, majortickspixeldist, scaleformat, levelhihi, levelhigh, levellow, levellolo, showhihi, showhigh
# showlow, showlolo
#class ScaledSlider(_Widget, _p._PVName, _p._Horizontal, _p._ForegroundColor, _p._BackgroundColor, _p._Transparent, _p._Font,
#                   _p._ShowScale, _p._ShowMinorTicks, _p._MajorTicksPixelDist, _p._ScaleFormat, _p._LevelsandShow, _p._AlarmBorder,
#                   _p._Increment, _p._MinMax, _p._LimitsFromPV, _p._Enabled):
#    pass

class Scrollbar(_Widget, _p._PVName, _p._Horizontal, _p._ShowValueTip, _p._AlarmBorder, _p._MinMax,
                _p._LimitsFromPV, _p._BarLength, _p._Increment, _p._Enabled):
    def __init__(self, name, pv_name, x, y, width, height):
        _Widget.__init__(self, 'scrollbar', name, x, y, width, height)
        self.pv_name(pv_name)

class SlideButton(_Widget, _p._PVName, _p._Bit, _p._Label, _p._OffColor, _p._OnColor, _p._Font, _p._ForegroundColor,
                  _p._AutoSize, _p._AlarmBorder, _p._Enabled, _p._Confirmation):
    def __init__(self, name, label, pv_name, x, y, width, height):
        _Widget.__init__(self, 'slide_button', name, x, y, width, height)
        self.pv_name(pv_name)
        self.label(label)

class Spinner(_Widget, _p._PVName, _p._Format, _p._Precision, _p._ShowUnits, _p._ForegroundColor, _p._BackgroundColor,
              _p._ButtonsOnLeft, _p._AlarmBorder, _p._MinMax, _p._LimitsFromPV, _p._Increment, _p._Enabled):
    def __init__(self, name, pv_name, x, y, width, height):
        _Widget.__init__(self, 'spinner', name, x, y, width, height)
        self.pv_name(pv_name)

class TextEntry(_Widget, _p._PVName, _p._Font, _p._ForegroundColor, _p._BackgroundColor, _p._Format,
                _p._Precision, _p._ShowUnits, _p._WrapWords, _p._MultiLine, _p._AlarmBorder, _p._Enabled,
                _p._Border):
    """ Text Entry widget, req: pv name """
    def __init__(self, name, pv_name, x, y, width, height):
        _Widget.__init__(self, 'textentry', name, x, y, width, height)
        self.pv_name(pv_name)

# Plots

class DataBrowser(_Widget, _p._Macro, _p._File, _p._ShowToolbar, _p._SelectionValuePV):
    def __init__(self, name, file, x, y, width, height):
        _Widget.__init__(self, 'databrowser', name, x, y, width, height)
        self.file(file)

# showtoolbar, colormap, colorbar, xaxis, yaxis, datawidth, interpolation, colormode, unsigneddata, autoscale,
# logscale, cursor, roi
#class Image(_Widget, _p._PVName, _p._ForegroundColor, _p._BackgroundColor, _p._ShowToolbar, _p._ColorMap, _p._ColorBar,
#            _p._XAxis, _p._YAxis, _p._AlarmBorder, _p._DataWidth, _p._Interpolation, _p._ColorMode, _p._UnsignedData,
#            _p._AutoScale, _p._LogScale, _p._MinMax, _p._Cursor, _p._RegionsOfInterest):
#    pass

# showgrid, title, labelfont, scalefont, showtoolbar, tooltip, timerange, yaxes, traces
#class StripChart(_Widget, _p._ForegroundColor, _p._BackgroundColor, _p._ShowGrid, _p._Title, _p._LabelFont, _p._ScaleFont,
#                 _p._ShowToolbar, _p._ToolTip, _p._TimeRange, _p._YAxes, _p._Traces):
#    pass


# gridcolor, title, showtoolbar, showlegend, xaxis, yaxes, traces, markers
#class XYPlot(_Widget, _p._ForegroundColor, _p._BackgroundColor, _p._GridColor, _p._Title, _p._ShowToolbar, _p._ShowLegend,
#             _p._XAxis, _p._YAxes, _p._Traces, _p._Markers):
#    pass

# Structure
class Array(_Widget, _p._PVName, _p._Macro, _p._ForegroundColor, _p._BackgroundColor, _p._AlarmBorder):
    def __init__(self, name, pv_name, x, y, width, height):
        _Widget.__init__(self, 'array', name, x, y, width, height)
        self.pv_name(pv_name)

class EmbeddedDisplay(_Widget, _p._Macro, _p._File, _p._ResizeBehavior, _p._GroupName, _p._Transparent, _p._Border):
    def __init__(self, name, file, x, y, width, height):
        _Widget.__init__(self, 'embedded', name, x, y, width, height)
        self.file(file)

class Group(_Widget, _p._Structure, _p._Macro, _p._Style, _p._Font, _p._ForegroundColor, _p._BackgroundColor, _p._Transparent):
    def __init__(self, name, x, y, width, height):
        _Widget.__init__(self, 'group', name, x, y, width, height)

# tab width, tab height, tab spacing, selected color, deselected color, active tab
#class NavigationTabs(_Widget, _p._Tabs, _p._Direction, _p._TabWidth, _p._TabHeight, _p._TabSpacing, _p._SelectColor, _p._Font, _p._ActiveTab):
#    pass


# tabs, active tab, direction, tab height
#class Tabs(_Widget, _p._Macro, _p._Tabs, _p._Font, _p._BackgroundColor, _p._ActiveTab, _p._Direction, _p._TabHeight):
#    pass

# Miscellaneous

class ThreeDViewer(_Widget, _p._File):
    def __init__(self, name, file, x, y, width, height):
        _Widget.__init__(self, '3dviewer', name, x, y, width, height)
        self.file(file)

class WebBrowser(_Widget, _p._URL, _p._ShowToolbar):
    def __init__(self, name, url, x, y, width, height):
        _Widget.__init__(self, 'webbrowser', name, x, y, width, height)
        self.url(url)


if __name__ == '__main__':
    pass




from xml.etree.ElementTree import Element, SubElement
from phoebusgen._shared_property_helpers import _SharedPropertyFunctions
from enum import Enum


class Property(object):
    def __init__(self, root_element):
        self.root = root_element
        self.formats_array = ['default', 'decimal', 'exponential', 'engineering', 'hexadecimal',
                              'compact', 'string',  'sexagesimal hh:mm:ss', 'sexagesimal hms 24h rad',
                              'sexagesimal dms 360deg rad', 'binary']
        self.shared_functions = _SharedPropertyFunctions(self.root)

    class RotationStep(Enum):
        zero = 0
        ninety = 1
        one_hundred_eighty = 2
        negative_ninety = 3

    class HorizontalAlignment(Enum):
        left = 0
        center = 1
        right = 2

    class VerticalAlignment(Enum):
        top = 0
        middle = 1
        bottom = 2

    class FontStyle(Enum):
        regular = 'REGULAR'
        italic = 'ITALIC'
        bold = 'BOLD'
        bold_and_italic = 'BOLD_ITALIC'

    class GroupStyle(Enum):
        group_box = 0
        title_bar = 1
        line = 2
        none = 3

    class Resize(Enum):
        no_resize = 0
        size_content_to_fit_widget = 1
        size_widget_to_match_content = 2
        stretch_content_to_fit_widget = 3
        crop_content = 4

    class FileComponent(Enum):
        full_path = 0
        directory = 1
        name_and_extension = 2
        base_name = 3

    class Mode(Enum):
        toggle = 0
        push = 1
        push_inverted = 2


    def add_pv_name(self, name):
        self.shared_functions.generic_property('pv_name', name)

    def add_precision(self, val):
        self.shared_functions.integer_property('precision', val)

    def add_show_units(self, show):
        self.shared_functions.boolean_property('show_units', show)

    def add_wrap_words(self, wrap):
        self.shared_functions.boolean_property('wrap_words', wrap)

    def add_transparent(self, transparent):
        self.shared_functions.boolean_property('transparent', transparent)

    def add_rotation(self, rotation):
        self.shared_functions.number_property('rotation', rotation)

    def add_rotation_step(self, rotation):
        if type(rotation) != self.RotationStep:
            print('The component parameter must be of type Rotation enum! Not: {}'.format(type(rotation)))
            return
        self.shared_functions.generic_property('rotation_step', rotation.value)

    def add_horizontal_alignment(self, alignment):
        if type(alignment) != self.HorizontalAlignment:
            print('The component parameter must be of type HorizontalAlignment enum! Not: {}'.format(type(alignment)))
            return
        self.shared_functions.generic_property('horizontal_alignment', alignment.value)

    def add_vertical_alignment(self, alignment):
        if type(alignment) != self.VerticalAlignment:
            print('The component parameter must be of type VerticalAlignment enum! Not: {}'.format(type(alignment)))
            return
        self.shared_functions.generic_property('vertical_alignment', alignment.value)

    def add_background_color(self, name, red, green, blue, alpha):
        e = self.shared_functions.create_element('background_color')
        self.shared_functions.create_color_element(e, name, red, green, blue, alpha)

    def add_foreground_color(self, name, red, green, blue, alpha):
        e = self.shared_functions.create_element('foreground_color')
        self.shared_functions.create_color_element(e, name, red, green, blue, alpha)

    def add_fill_color(self, name, red, green, blue, alpha):
        e = self.shared_functions.create_element('fill_color')
        self.shared_functions.create_color_element(e, name, red, green, blue, alpha)

    def add_empty_color(self, name, red, green, blue, alpha):
        e = self.shared_functions.create_element('empty_color')
        self.shared_functions.create_color_element(e, name, red, green, blue, alpha)

    def add_named_font(self, name):
        self.shared_functions.create_named_font_elemet(name)

    def get_font_element(self):
        font_root_elem = self.root.find('font')
        if font_root_elem is None:
            font_root_elem = self.shared_functions.create_element('font')
            self.root.append(font_root_elem)
        child_font_elem = font_root_elem.find('font')
        if child_font_elem is None:
            child_font_elem = Element('font')
            child_font_elem.attrib = {'family': 'Liberation Sans', 'size': '14', 'style': 'REGULAR'}
            font_root_elem.append(child_font_elem)
        return child_font_elem

    def add_font_family(self, val):
        child_elem = self.get_font_element()
        child_elem.attrib['family'] = str(val)

    def add_font_style(self, val):
        if type(val) != self.FontStyle:
            print('The font style parameter must be of type FontStyle enum! Not: {}'.format(type(val)))
            return
        child_elem = self.get_font_element()
        child_elem.attrib['style'] = val.value

    def add_font_size(self, val):
        if type(val) == int or type(val) == float:
            child_elem = self.get_font_element()
            child_elem.attrib['size'] = str(int(val))
        else:
            print('Font size must be a number! Not: {}'.format(val))

    def add_border_width(self, width):
        self.shared_functions.integer_property('border_width', width)

    def add_border_color(self, name, red, green, blue, alpha):
        e = self.shared_functions.create_element('border_color')
        self.shared_functions.create_color_element(e, name, red, green, blue, alpha)

    def add_format(self, format_val):
        val = str(format_val).lower()
        try:
            v = self.formats_array.index(val)
        except ValueError:
            print('Invalid format. Given format {}'.format(format))
            return
        self.shared_functions.generic_property('format', v)

    def add_text(self, text):
        self.shared_functions.generic_property('text', text)

    def add_auto_size(self, auto):
        self.shared_functions.boolean_property('auto_size', auto)

    def add_multi_line(self, val):
        self.shared_functions.boolean_property('multi_line', val)

    def add_macro(self, name, val):
        self.shared_functions.add_macro(name, val)

    def add_bit(self, val):
        self.shared_functions.integer_property('bit', val)

    def add_square(self, val):
        self.shared_functions.boolean_property('square', val)

    def add_labels_from_pv(self, val):
        self.shared_functions.boolean_property('labels_from_pv', val)

    def add_off_color(self, name, red, green, blue, alpha):
        e = self.shared_functions.create_element('off_color')
        self.shared_functions.create_color_element(e, name, red, green, blue, alpha)

    def add_off_label(self, label):
        self.shared_functions.generic_property('off_label', label)

    def add_on_color(self, name, red, green, blue, alpha):
        e = self.shared_functions.create_element('on_color')
        self.shared_functions.create_color_element(e, name, red, green, blue, alpha)

    def add_on_label(self, label):
        self.shared_functions.generic_property('on_label', label)

    def add_line_color(self, name, red, green, blue, alpha):
        e = self.shared_functions.create_element('line_color')
        self.shared_functions.create_color_element(e, name, red, green, blue, alpha)

    def add_on_image(self, image_file):
        self.shared_functions.generic_property('on_image', image_file)

    def add_off_image(self, image_file):
        self.shared_functions.generic_property('off_image', image_file)

    def add_alarm_border(self, val):
        self.shared_functions.boolean_property('border_alarm_sensitive', val)

    def add_enabled(self, val):
        self.shared_functions.boolean_property('enabled', val)

    def add_confirmation_dialog(self, val):
        self.shared_functions.boolean_property('show_confirmation_dialog', val)

    def add_confirmation_message(self, message):
        self.shared_functions.generic_property('confirm_message', message)

    def add_password(self, password):
        self.shared_functions.generic_property('password', password)

    def add_corner_width(self, width):
        self.shared_functions.integer_property('corner_width', width)

    def add_corner_height(self, height):
        self.shared_functions.integer_property('corner_height', height)

    def add_line_width(self, width):
        self.shared_functions.integer_property('line_width', width)

    def add_angle_start(self, val):
        self.shared_functions.number_property('start_angle', val)

    def add_angle_size(self, val):
        self.shared_functions.number_property('total_angle', val)

    def add_file(self, val):
        self.shared_functions.generic_property('file', val)

    def add_stretch_to_fit(self, val):
        self.shared_functions.boolean_property('stretch_image', val)

    def add_rotation(self, val):
        self.shared_functions.number_property('rotation', val)

    def add_resize_behavior(self, resize):
        if type(resize) != self.Resize:
            print('Resize behavior input must be of type enum Resize. Not: {}'.format(type(resize)))
        self.shared_functions.generic_property('resize', resize.value)

    def add_group_name(self, val):
        self.shared_functions.generic_property('group_name', val)

    def add_style(self, style):
        if type(style) != self.GroupStyle:
            print('Input type for param to group style must be of type GroupStyle. Not: {}'.format(type(style)))
            return
        self.shared_functions.generic_property('style', style.value)

    def add_action(self, action_type, description, args):
        root_action = self.root.find('actions')
        if root_action is None:
            root_action = SubElement(self.root, 'actions')
        action = SubElement(root_action, 'action')
        action.attrib['type'] = action_type
        sub = SubElement(action, 'description')
        sub.text = str(description)
        for arg, val in args.items():
            sub = SubElement(action, arg)
            sub.text = str(val)

    def add_action_execute_as_one(self, val):
        if type(val) == bool:
            action = str(val).lower()
        elif type(val) == int:
            action = str(bool(val)).lower()
        elif val.lower() == 'true' or val.lower() == 'false':
            action = val.lower()
        else:
            print('action_execute_as_one must take a boolean value! Not: {}'.format(val))
            return
        root_action = self.root.find('actions')
        if root_action is None:
            root_action = SubElement(self.root, 'actions')
        root_action.attrib['execute_as_one'] = action

    def add_label(self, val):
        self.shared_functions.generic_property('label', val)

    def add_horizontal(self, val):
        self.shared_functions.boolean_property('horizontal', val)

    def add_item(self, val):
        root_item = self.root.find('items')
        if root_item is None:
            root_item = SubElement(self.root, 'items')
        sub = SubElement(root_item, 'item')
        sub.text = val

    def add_items_from_pv(self, val):
        self.shared_functions.boolean_property('items_from_pv', val)

    def add_minimum(self, val):
        self.shared_functions.number_property('minimum', val)

    def add_maximum(self, val):
        self.shared_functions.number_property('maximum', val)

    def add_limits_from_pv(self, val):
        self.shared_functions.boolean_property('limits_from_pv', val)

    def add_scale_visible(self, val):
        self.shared_functions.boolean_property('scale_visible', val)

    def add_url(self, url):
        self.shared_functions.generic_property('url', url)

    def add_show_toolbar(self, val):
        self.shared_functions.boolean_property('show_toolbar', val)

    def add_buttons_on_left(self, val):
        self.shared_functions.boolean_property('buttons_on_left', val)

    def add_increment(self, val):
        self.shared_functions.number_property('increment', val)

    def add_file_component(self, val):
        if type(val) != self.FileComponent:
            print('The component parameter must be of type FileComponent enum! Not: {}'.format(type(val)))
            return
        self.shared_functions.generic_property('component', val.value)

    def add_editable(self, val):
        self.shared_functions.boolean_property('editable', val)

    def add_selected_color(self, name, red, green, blue, alpha):
        e = self.shared_functions.create_element('selected_color')
        self.shared_functions.create_color_element(e, name, red, green, blue, alpha)

    def add_show_led(self, val):
        self.shared_functions.generic_property('show_led', val)

    def add_mode(self, val):
        if type(val) != self.Mode:
            print('The component parameter must be of type Mode enum! Not: {}'.format(type(val)))
            return
        self.shared_functions.generic_property('mode', val.value)

    def add_show_value(self, val):
        self.shared_functions.boolean_property('show_value', val)

    def add_show_limits(self, val):
        self.shared_functions.boolean_property('show_limits', val)

    def add_needle_color(self, name, red, green, blue, alpha):
        e = self.shared_functions.create_element('needle_color')
        self.shared_functions.create_color_element(e, name, red, green, blue, alpha)

    def add_knob_color(self, name, red, green, blue, alpha):
        e = self.shared_functions.create_element('knob_color')
        self.shared_functions.create_color_element(e, name, red, green, blue, alpha)

    def add_show_value_tip(self, val):
        self.shared_functions.boolean_property('show_value_tip', val)

    def add_bar_length(self, val):
        self.shared_functions.number_property('bar_length', val)

    def add_selection_value_pv(self, val):
        self.shared_functions.generic_property('selection_value_pv', val)


from xml.etree.ElementTree import Element, SubElement, tostring
from xml.dom import minidom
from phoebusgen.widget.property import Property
from phoebusgen._shared_property_helpers import _SharedPropertyFunctions

class _Widget(object):
    def __init__(self, w_type, name, x_pos, y_pos, width, height):
        self.root = Element('widget', type=w_type, version='2.0.0')
        name_child = SubElement(self.root, 'name')
        name_child.text = name

        self.shared_functions = _SharedPropertyFunctions(self.root)
        self.shared_functions.integer_property('x', x_pos)
        self.shared_functions.integer_property('y', y_pos)
        self.shared_functions.integer_property('width', width)
        self.shared_functions.integer_property('height', height)

        self._prop_factory = Property(self.root)

    def visible(self, visible):
        child = SubElement(self.root, 'visible')
        child.text = str(visible)

    def version(self, version):
        self.root.attrib['version'] = version

    def name(self, val):
        self.shared_functions.generic_property('name', val)

    def width(self, val):
        self.shared_functions.integer_property('width', val)

    def height(self, val):
        self.shared_functions.integer_property('height', val)

    def x(self, val):
        self.shared_functions.integer_property('x', val)

    def y(self, val):
        self.shared_functions.integer_property('y', val)

    #def class_name(self, name):
    #    pass

    #def rule(self, rule):
    #    pass

    #def scripts(self, script):
    #    pass

    def tool_tip(self, tool_tip):
        child = SubElement(self.root, 'tooltip')
        child.text = tool_tip

    def find_element(self, tag):
        elements = self.root.findall(tag)
        # check to make sure there are not more than 1 elements
        # we don't want duplicate tags
        if len(elements) > 1:
            print('Warning, more than one element of the same tag! Returning a list')
            return elements
        elif len(elements) == 0:
            return None
        else:
            return elements[0]

    def remove_element(self, tag):
        element = self.find_element(tag)
        if element is not None:
            self.root.remove(element)

    def get_element_value(self, tag):
        return self.find_element(tag).text

    # From: https://pymotw.com/3/xml.etree.ElementTree/create.html
    def prettify(self, elem):
        """Return a pretty-printed XML string for the Element.
        """
        rough_string = tostring(elem, 'utf-8')
        reparse_xml = minidom.parseString(rough_string)
        return reparse_xml.toprettyxml(indent="  ", newl="\n")

    def __str__(self):
        return self.prettify(self.root)

    def __repr__(self):
        return self.prettify(self.root)


if __name__ == '__main__':
    pass
